#  Copyright 2023 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#  SPDX-License-Identifier: Apache-2.0

COMPLETED = "COMPLETED"
NOT_SET = "NOT_SET"
ERRORED = "ERRORED"
QUEUE_STATUS = "QUEUE_STATUS"
IN_PROGRESS = "IN_PROGRESS"
BLOCKED = "BLOCKED"
RESOURCES_REQUIRED = "resources_required"

CONTROL_EVENT__COMPLETE = "CONTROL_EVENT__COMPLETE"
