#   Copyright 2023 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#   SPDX-License-Identifier: Apache-2.0


MANIFEST_SECTION_NAMES = "manifest_section_names"
MANIFEST_ITEM_NAMES = "manifest_item_names"
MANIFEST_ACCOUNT_IDS = "manifest_account_ids"
